use chrono::{DateTime, Utc};
use serde::{Deserialize, Serialize};

#[derive(Serialize, Deserialize, Default, Debug, Clone)]
pub struct Filter {
    #[serde(default, skip_serializing_if = "Vec::is_empty")]
    pub ids: Vec<super::Addr>,
    #[serde(default, skip_serializing_if = "Vec::is_empty")]
    pub authors: Vec<super::Addr>,
    #[serde(default, skip_serializing_if = "Vec::is_empty")]
    pub kinds: Vec<u32>,
    #[serde(default, skip_serializing_if = "Vec::is_empty", rename = "#e")]
    pub references_to_event: Vec<super::Addr>,
    #[serde(default, skip_serializing_if = "Vec::is_empty", rename = "#p")]
    pub references_to_public_key: Vec<super::Addr>,
    #[serde(
        default,
        with = "super::option_ts_seconds",
        skip_serializing_if = "Option::is_none"
    )]
    pub since: Option<DateTime<Utc>>,
    #[serde(
        default,
        with = "super::option_ts_seconds",
        skip_serializing_if = "Option::is_none"
    )]
    pub until: Option<DateTime<Utc>>,
    #[serde(default, skip_serializing_if = "is_zero")]
    pub limit: u64,
}

#[inline]
fn is_zero(number: &u64) -> bool {
    *number == 0
}

#[cfg(test)]
mod test {
    use super::*;

    #[test]
    fn deserialize() {
        let json = "{\"authors\":[\"c0f0f1\", \"1234567901\"]}";
        let obj: Result<Filter, _> = serde_json::from_str(&json);
        assert!(obj.is_ok());
    }

    #[test]
    fn serialize() {
        let now = Utc::now();
        let x = Filter {
            since: Some(now),
            ..Default::default()
        };

        assert_eq!(
            format!("{}\"since\":{:?}{}", "{", now.timestamp(), "}"),
            serde_json::to_string(&x).expect("valid json")
        );
    }
}
