use super::Content;
use serde::{
    de::{self, Deserializer},
    ser::{self, Serializer},
    Deserialize,
};

#[derive(Debug, PartialEq, Eq, Clone, Copy)]
pub enum Kind {
    Metadata,
    ShortTextNote,
    RecommendRelayer,
    Contacts,
    EncryptedDirectMessages,
    Unknown(u32),
}

impl From<Kind> for u32 {
    fn from(kind: Kind) -> Self {
        match kind {
            Kind::Metadata => 0,
            Kind::ShortTextNote => 1,
            Kind::RecommendRelayer => 2,
            Kind::Contacts => 3,
            Kind::EncryptedDirectMessages => 4,
            Kind::Unknown(t) => t,
        }
    }
}

impl From<u32> for Kind {
    fn from(kind: u32) -> Self {
        match kind {
            0 => Kind::Metadata,
            1 => Kind::ShortTextNote,
            2 => Kind::RecommendRelayer,
            3 => Kind::Contacts,
            4 => Kind::EncryptedDirectMessages,
            any => Kind::Unknown(any),
        }
    }
}

impl ser::Serialize for Kind {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_u32((*self).into())
    }
}

impl<'de> de::Deserialize<'de> for Kind {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: Deserializer<'de>,
    {
        let raw: u32 = Deserialize::deserialize(deserializer)?;
        Ok(raw.into())
    }
}

impl From<&Content> for Kind {
    fn from(content: &Content) -> Self {
        match content {
            Content::Metadata(_) => Kind::Metadata,
            Content::ShortTextNote(_) => Kind::ShortTextNote,
            Content::RecommendRelayer(_) => Kind::RecommendRelayer,
            Content::Contacts(_) => Kind::Contacts,
            Content::EncryptedDirectMessage(_) => Kind::EncryptedDirectMessages,
            Content::Unknown(kind, _) => *kind,
        }
    }
}
